#!/usr/bin/env bash
###
 # Install and set up for MySQL/MariaDB.
 ##


# Import utils script.
. $(dirname ${0})/utils.sh


# Stop on error.
set -e


###
 # Script entry point.
 # Checks that expected args were provided, to indicate what database type is being installed to what OS.
 # Then calls the corresponding function to actually execute logic.
 ##
function main () {

    check_is_user root

    # Check that expected os key exists.
    if [[ ! ${!kwargs[@]} =~ "os" ]]
    then
        echo -e "${text_red}Failed to provide expected OS kwarg. Terminating script.${text_reset}"
        exit 1
    fi

    # Check that expected db key exists.
    if [[ ! ${!kwargs[@]} =~ "db" ]]
    then
        echo -e "${text_red}Failed to provide expected database kwarg. Terminating script.${text_reset}"
        exit 1
    fi

    # Check os to install for.
    if [[ ${kwargs["os"]} == "arch" ]]
    then
        # Handle for Arch Linux.
        if [[ ${kwargs["db"]} == "mariadb" ]]
        then
            install_arch_mariadb
        else
            echo -e "${text_red}Unrecognized database type. Terminating script.${text_reset}"
            exit 1
        fi

    elif [[ ${kwargs["os"]} == "ubuntu" ]]
    then
        # Handle for Ubuntu.
        if [[ ${kwargs["db"]} == "mysql" ]]
        then
            install_ubuntu_mysql

        elif [[ ${kwargs["db"]} == "mariadb" ]]
        then
            install_ubuntu_mariadb
        else
            echo -e "${text_red}Unrecognized database type. Terminating script.${text_reset}"
            exit 1
        fi

    else
        echo -e "${text_red}Unrecognized OS type. Terminating script.${text_reset}"
        exit 1
    fi

    set_config

}


###
 # Installation logic for Arch Linux systems.
 ##
function install_arch_mariadb () {

    # Install MariaDB.
    pacman -Syu mariadb --noconfirm > /dev/null

    # Run initial configuration.
    sudo mariadb-install-db --user=mysql --basedir=/usr --datadir=/var/lib/mysql > /dev/null

    # Start MariaDB service and set to start on boot.
    systemctl enable mariadb.service > /dev/null
    systemctl start mariadb.service > /dev/null

    # Proceed with initial configuration.
    mysql_secure_installation
}


###
 # MySQL installation logic for Ubuntu systems.
 ##
function install_ubuntu_mysql () {
    echo -e "${text_blue}Installing MySQL for Ubuntu Linux.${text_reset}"
    echo -e "${text_blue}Sudo is required for installing/updating system files.${text_reset}"
    echo -e "${text_blue}This may ask for your system password.${text_reset}"

    # Install MySQL.
    sudo apt-get update > /dev/null 2>&1
    sudo apt-get install mysql-server mysql-client -y

    # Start MariaDB service and set to start on boot.
    systemctl enable mysql.service > /dev/null
    systemctl start mysql.service > /dev/null

    echo ""
    get_user_confirmation "Allow password login with root user? Recommended for local dev/testing setups."
    if [[ ${return_value} == true ]]
    then
        # Get desired password to use for root login.
        user_input=""
        while [[ "${user_input}" == "" ]]
        do
            echo -e "${text_cyan}Enter desired password:${text_reset}"
            read user_input
        done

        # Update root account to login with password.
        sudo mysql -u root -e "ALTER USER 'root'@'localhost' IDENTIFIED WITH mysql_native_password BY '${user_input}'; FLUSH PRIVILEGES;"
    fi

    echo ""
    echo ""
    echo -e "${text_blue}Initial MySQL setup complete.${text_reset}"
    echo -e "${text_blue}Now running additional configuration. This will give a few more prompts...${text_reset}"
    echo ""

    sleep 3

    # Proceed with initial configuration.
    mysql_secure_installation

    # Optionally install programming dependencies.
    echo ""
    get_user_confirmation "Install programming dependencies? These are necessary for things like serving Django projects."
    if [[ ${return_value} == true ]]
    then
        sudo apt-get install libmysqlclient-dev -y
    fi
}


###
 # MariaDB installation logic for Ubuntu systems.
 ##
function install_ubuntu_mariadb () {
    echo -e "${text_blue}Installing MariaDB for Ubuntu Linux.${text_reset}"
    echo -e "${text_blue}Sudo is required for installing/updating system files.${text_reset}"
    echo -e "${text_blue}This may ask for your system password.${text_reset}"

    # Install MariaDB.
    sudo apt-get update > /dev/null 2>&1
    sudo apt-get install mariadb-server -y > /dev/null

    # Start MariaDB service and set to start on boot.
    systemctl enable mariadb.service > /dev/null
    systemctl start mariadb.service > /dev/null

    # Proceed with initial configuration.
    mysql_secure_installation

    # Optionally install programming dependencies.
    echo ""
    get_user_confirmation "Install programming dependencies? These are necessary for things like serving Django projects."
    if [[ ${return_value} == true ]]
    then
        sudo apt-get install libmariadbclient-dev libssl-dev -y > /dev/null
    fi
}


###
 # Updates config file for first time setup.
 # Should apply to all currently handled OS systems and database types.
 ##
function set_config () {
    # Check that expected config file exists.
    if [[ -f "/etc/mysql/my.cnf" ]]
    then
        config_file="/etc/mysql/my.cnf"
    elif [[ -f "/etc/my.cnf" ]]
    then
        config_file="/etc/my.cnf"
    else
        echo -e "${text_orange}Failed to locate config file at \"/etc/mysql/my.cnf\" or \"/etc/my.cnf\".${text_reset}"
        echo -e "${text_orange}Unable to set config for first time setup.${text_reset}"
        return
    fi

    # Check file contents.
    file_contents=$(cat ${config_file})
    search_string_1="default-character-set ="
    if [[ ${kwargs["db"]} == "mysql" ]]
    then
        search_string_2="collation-server ="
        search_string_3="character-set-server ="
    else
        search_string_2="collation_server ="
        search_string_3="character_set_server ="
    fi
    default_char_set_found=false
    if [[ ! "${file_contents}" =~ "${search_string_1}" ||
          ! "${file_contents}" =~ "${search_string_2}" ||
          ! "${file_contents}" =~ "${search_string_3}" ]]
    then
        # One or more values not set.
        echo -e "${text_blue}Running config first time setup.${text_reset}"

        # Set config header.
        echo "" >> ${config_file}
        echo "" >> ${config_file}
        echo "###" >> ${config_file}
        echo " # Automatically set Config Values" >> ${config_file}
        echo " ##" >> ${config_file}

        # Check for default char set.
        if [[ "${file_contents}" =~ "${search_string_1}" ]]
        then
            default_char_set_found=true
        fi

        # Set default char set if was not found.
        if [[ ${default_char_set_found} == false ]]
        then
            # Default char set not found. Define here for client.
            echo "[client]" >> ${config_file}
            echo "${search_string_1} utf8mb4" >> ${config_file}
            echo "" >> ${config_file}
        fi

        # Check for default collation and server set.
        if [[ ! "${file_contents}" =~ "${search_string_2}" || ! "${file_contents}" =~ "${search_string_3}" ]]
        then
            # Default collation or server set not found. Define here.
            echo "[mysqld]" >> ${config_file}

            if [[ ! "${file_contents}" =~ "${search_string_2}" ]]
            then
                echo "${search_string_2} utf8mb4_unicode_ci" >> ${config_file}
            fi
            if [[ ! "${file_contents}" =~ "${search_string_3}" ]]
            then
                echo "${search_string_3} utf8mb4" >> ${config_file}
            fi
            echo "" >> ${config_file}
        fi

        # Set default char set if was not found.
        if [[ ${default_char_set_found} == false ]]
        then
            # Default char set not found. Define here for mysql.
            echo "[mysql]" >> ${config_file}
            echo "${search_string_1} utf8mb4" >> ${config_file}
            echo "" >> ${config_file}

            # While here, also set auto complete within the client.
            echo "# Enable auto-completion in MySQL client." >> ${config_file}
            echo "auto-rehash" >> ${config_file}
            echo "" >> ${config_file}
        fi

        # Restart database for new config values.
        if [[ ${kwargs["db"]} == "mysql" ]]
        then
            systemctl restart mysql.service
        else
            systemctl restart mariadb.service
        fi

    else
        # Config values found.
        echo -e "${text_orange}Config appears to already be set. Skipping config first time setup.${text_reset}"
    fi

}


main
